@@ -1,7 +1,7 @@
 import os
 import re
 
-from backend import rrdbackend
+from pyrrd.backend import external
 
 def validateVName(name):
     '''
@@ -143,7 +143,7 @@ class DataDefinition(object):
     def __init__(self, vname='', rrdfile='', dsName='', cdef='AVERAGE',
         step=None, start=None, end=None, reduce=None):
         self.vname = validateVName(vname)
-        self.rrdfile = rrdfile
+        self.rrdfile = escapeColons(rrdfile)
         self.dsName = dsName
         self.cdef = cdef
         self.step = step
@@ -353,8 +353,7 @@ class Print(object):
         Time representations must have their ':'s escaped, since
         the colon is the RRDTool separator for parameters.
         '''
-        main = self.abbr+':%(vname)s:"%(format)s"' % (
-            self.__dict__)
+        main = self.abbr+':%s:"%s"' % (self.vname, escapeColons(self.format))
         return main
 
 PRINT = Print
@@ -409,7 +408,7 @@ class GraphComment(object):
         newLine = '\n'
         if not self.autoNewline:
             newLine = ''
-        main = 'COMMENT:"%s%s"' % ( self.comment, newLine)
+        main = 'COMMENT:"%s"%s' % ( self.comment, newLine)
         return main
 
 COMMENT = GraphComment
@@ -496,7 +495,7 @@ class Line(object):
         '''
         main = self.abbr
         if self.width:
-            main += str(self.width)
+            main += unicode(self.width)
         main += ':%s' % self.vname
         if self.color:
             main += self.color
@@ -543,8 +542,51 @@ class GraphTick(object):
     the length of the tick mark as a fraction of the y-axis; the
     default value is 0.1 (10% of the axis). Note that the color
     specification is not optional.
+
+    >>> def1 = DEF(rrdfile='/home/rrdtool/data/router1.rrd',
+    ...   vname='ds0a', dsName='ds0')
+
+    >>> GraphTick(def1,'#ffffff',0.3,'Alarm!')
+    TICK:ds0a#ffffff:0.3:"Alarm!"
     '''
-    # XXX TODO
+    def __init__(self, defObj=None, color=None, fraction=None, legend=''):
+        if not defObj:
+            raise Exception, "You must provide either a value " + \
+                "or a definition object."
+        else:
+            value1 = defObj.vname
+
+        if fraction:
+            if not (isinstance(fraction, float) or isinstance(fraction, int)):
+                raise TypeError, "The parameter 'fraction' must" + \
+                        "be a float value between 0 and 1."
+            else:
+                if 0 <= fraction <= 1:
+                    value2 = fraction
+                else:
+                    raise ValueError, "The parameter 'fraction' must" + \
+                        "be a value between 0 and 1."
+        if not color:
+            raise ValueError, "Missing required parameter color"
+
+
+        self.vname = value1
+        self.color = color
+        self.fraction = value2
+        self.legend = legend
+        self.abbr = 'TICK'
+
+    def __repr__(self):
+        main = self.abbr
+        main += ':%s' % self.vname
+        if self.color:
+            main += self.color
+        if self.fraction:
+            main += ':%s' % self.fraction
+        if self.legend:
+            main += ':"%s"' % self.legend
+        return main
+
 TICK = GraphTick
 
 class GraphShift(object):
@@ -675,10 +717,12 @@ class Graph(object):
     to graph it (or print it). This ends the rrdtool graph sequence.
 
     # Let's create and RRD file and dump some data in it
+    >>> import tempfile
     >>> from rrd import RRD, RRA, DS
     >>> dss = []
     >>> rras = []
-    >>> filename = '/tmp/test.rrd'
+    >>> tfile = tempfile.NamedTemporaryFile()
+    >>> filename = tfile.name
     >>> ds1 = DS(dsName='speed', dsType='COUNTER', heartbeat=600)
     >>> dss.append(ds1)
     >>> rra1 = RRA(cf='AVERAGE', xff=0.5, steps=1, rows=24)
@@ -730,16 +774,14 @@ class Graph(object):
     >>> ca.arrow = '#FFFFFF'
 
     # Now that we've got everything set up, let's make a graph
-    >>> graphfile = '/tmp/myspeed.png'
-    >>> g = Graph(graphfile, start=920805000, end=920810000,
-    ...   vertical_label='km/h', color=ca)
+    >>> graphfile = tempfile.NamedTemporaryFile()
+    >>> g = Graph(graphfile.name, start=920805000, end=920810000,
+    ...   vertical_label='km/h', color=ca, imgformat='png')
     >>> g.data.extend([def1, cdef1, cdef2, cdef3, vdef1, vdef2, line1,
     ...   area1, area2, line2, gprint1])
     >>> g.write()
-    >>> os.path.exists(graphfile)
+    >>> os.path.exists(graphfile.name)
     True
-    >>> os.unlink(filename)
-    >>> os.unlink(graphfile)
     '''
     # Note that we don't use the Twisted camel case convention for the
     # parameters in the following method signature due to the fact that these
@@ -754,11 +796,13 @@ class Graph(object):
         zoom=None, font=None, font_render_mode=None,
         font_smoothing_threshold=None, slope_mode=None,
         imgformat='', interlaced=False, no_legend=False,
-        force_rules_legend=False, tabwidth=None, base=None):
+        force_rules_legend=False, tabwidth=None, base=None,
+        backend=external):
 
         self.filename = filename
         if not imgformat:
-            fn, ext = filename.split(os.extsep)
+            fn, ext = os.path.splitext(filename)
+            ext = ext.strip(os.extsep)
             imgformat = ext
         self.imgformat = validateImageFormat(imgformat)
 
@@ -790,6 +834,8 @@ class Graph(object):
         self.force_rules_legend = force_rules_legend
         self.tabwidth = tabwidth
         self.base = base
+        self.slope_mode = slope_mode
+        self.backend = backend
 
         if filename.strip() == '-':
             # send to stdout
@@ -799,14 +845,12 @@ class Graph(object):
     def write(self, debug=False):
         '''
         '''
-        data = rrdbackend.prepareObject('graph', self)
-        if debug: print data
-        rrdbackend.graph(*data)
+        data = self.backend.prepareObject('graph', self)
+        if debug:
+            print data
+        self.backend.graph(*data)
 
-def _test():
-    from doctest import testmod
-    return testmod()
 
 if __name__ == '__main__':
-    _test()
-
+    import doctest
+    doctest.testmod()
