@@ -1,8 +1,8 @@
 display_name = "PyRRD"
 library_name = "pyrrd"
-version = "0.0.7"
+version = "0.1.0"
 author = "Duncan McGreggor"
 author_email = "duncan@canonical.com"
 license = "BSD"
 url = "http://code.google.com/p/pyrrd/"
-description = "An Object-Oriented Python Interface for RRD"
+description = "An Object-Oriented Python Interface for RRDTool"
