@@ -3,7 +3,7 @@ from datetime import datetime
 
 from pyrrd import mapper
 from pyrrd import util
-from pyrrd.backend import rrdbackend
+from pyrrd.backend import external
 
 
 def validateDSName(name):
@@ -55,7 +55,7 @@ def validateRRACF(consolidationFunction):
     """
     cf = consolidationFunction.upper()
     valid = ['AVERAGE', 'MIN', 'MAX', 'LAST', 'HWPREDICT', 'SEASONAL',
-        'DEVSEASONAL', 'DEVPREDICT', 'FAILURES']
+             'DEVSEASONAL', 'DEVPREDICT', 'FAILURES']
     if cf in valid:
         return cf
     else:
@@ -66,17 +66,19 @@ def validateRRACF(consolidationFunction):
 
 class RRD(mapper.RRDMapper):
     """
+    >>> import os, tempfile
+    >>>
     >>> dss = []
     >>> rras = []
-    >>> filename = '/tmp/test.rrd'
+    >>> rrdfile = tempfile.NamedTemporaryFile()
     >>> dss.append(DataSource(dsName='speed', dsType='COUNTER', heartbeat=600))
     >>> rras.append(RRA(cf='AVERAGE', xff=0.5, steps=1, rows=24))
     >>> rras.append(RRA(cf='AVERAGE', xff=0.5, steps=6, rows=10))
-    >>> rrd = RRD(filename, ds=dss, rra=rras, start=920804400)
+    >>> rrd = RRD(rrdfile.name, ds=dss, rra=rras, start=920804400)
     >>> rrd.create()
-    >>> import os
-    >>> os.path.exists(filename)
+    >>> os.path.exists(rrdfile.name)
     True
+
     >>> rrd.bufferValue('920805600', '12363')
     >>> rrd.bufferValue('920805900', '12363')
     >>> rrd.bufferValue('920806200', '12373')
@@ -93,12 +95,10 @@ class RRD(mapper.RRDMapper):
     >>> rrd.update()
     >>> len(rrd.values)
     0
-    >>> os.unlink(filename)
-    >>> os.path.exists(filename)
-    False
     """
-    def __init__(self, filename=None, start=None, step=300, ds=[], rra=[],
-                 mode="w"):
+    def __init__(self, filename=None, start=None, step=300, ds=None, rra=None,
+                 mode="w", backend=external):
+        super(RRD, self).__init__()
         if filename == None:
             raise ValueError, "You must provide a filename."
         self.filename = filename
@@ -106,18 +106,73 @@ class RRD(mapper.RRDMapper):
             self.start = util.epoch(start)
         else:
             self.start = start
+        if not ds:
+            ds = []
+        if not rra:
+            rra = []
         self.ds = ds
         self.rra = rra
         self.values = []
         self.step = step
         self.lastupdate = None
-        if mode == "r":
+        self.mode = mode
+        # the backend attribute needs to be defined before the load call, since
+        # the load method (super class) expects the backend attribute
+        self.backend = backend
+        if self.mode == "r":
             self.load()
 
+    def bufferValue(self, timeOrData, *values):
+        """
+        The parameter 'values' can either be a an n-tuple, but it
+        is assumed that the order in which the values are sent is
+        the order in which they will be applied to the DSs (i.e.,
+        respectively... i.e., in the order that the DSs were added
+        to the RRD).
+
+        >>> my_rrd = RRD('somefile')
+        >>> my_rrd.bufferValue('1000000', 'value')
+        >>> my_rrd.update(debug=True, dryRun=True)
+        ('somefile', [u'1000000:value'])
+        >>> my_rrd.update(template='ds0', debug=True, dryRun=True)
+        ('somefile', ['--template', u'ds0', u'1000000:value'])
+        >>> my_rrd.values = []
+
+        >>> my_rrd.bufferValue('1000000:value')
+        >>> my_rrd.update(debug=True, dryRun=True)
+        ('somefile', ['1000000:value'])
+        >>> my_rrd.update(template='ds0', debug=True, dryRun=True)
+        ('somefile', ['--template', u'ds0', '1000000:value'])
+        >>> my_rrd.values = []
+
+        >>> my_rrd.bufferValue('1000000', 'value1', 'value2')
+        >>> my_rrd.bufferValue('1000001', 'value3', 'value4')
+        >>> my_rrd.update(debug=True, dryRun=True)
+        ('somefile', [u'1000000:value1:value2', u'1000001:value3:value4'])
+        >>> my_rrd.update(template=u'ds1:ds0', debug=True, dryRun=True)
+        ('somefile', ['--template', u'ds1:ds0', u'1000000:value1:value2', u'1000001:value3:value4'])
+        >>> my_rrd.values = []
+
+        >>> my_rrd.bufferValue('1000000:value')
+        >>> my_rrd.bufferValue('1000001:anothervalue')
+        >>> my_rrd.update(debug=True, dryRun=True)
+        ('somefile', ['1000000:value', '1000001:anothervalue'])
+        >>> my_rrd.update(template='ds0', debug=True, dryRun=True)
+        ('somefile', ['--template', u'ds0', '1000000:value', '1000001:anothervalue'])
+        >>> my_rrd.values = []
+        """
+        values = ':'.join([unicode(x) for x in values])
+        self.values.append((timeOrData, values))
+        self.lastupdate = float(unicode(timeOrData).split(":")[0])
+
+    # for backwards compatibility
+    bufferValues = bufferValue
+
     def create(self, debug=False):
-        data = rrdbackend.prepareObject('create', self)
-        if debug: print data
-        rrdbackend.create(*data)
+        data = self.backend.prepareObject('create', self)
+        if debug:
+            print data
+        self.backend.create(*data)
 
     # XXX this can be uncommented when we're doing full database imports with
     # the loads method and storing those values in the python objects
@@ -131,52 +186,6 @@ class RRD(mapper.RRDMapper):
     #            self.bufferValue(time, data)
     #        self.update()
 
-    def bufferValue(self, timeOrData, *values):
-        """
-        The parameter 'values' can either be a an n-tuple, but it
-        is assumed that the order in which the values are sent is
-        the order in which they will be applied to the DSs (i.e.,
-        respectively... i.e., in the order that the DSs were added
-        to the RRD).
-
-        >>> my_rrd = RRD('somefile')
-        >>> my_rrd.bufferValue('1000000', 'value')
-        >>> my_rrd.update(debug=True, dryRun=True)
-        ('somefile', ' 1000000:value')
-        >>> my_rrd.update(template='ds0', debug=True, dryRun=True)
-        ('somefile', '--template ds0 1000000:value')
-        >>> my_rrd.values = []
-
-        >>> my_rrd.bufferValue('1000000:value')
-        >>> my_rrd.update(debug=True, dryRun=True)
-        ('somefile', ' 1000000:value')
-        >>> my_rrd.update(template='ds0', debug=True, dryRun=True)
-        ('somefile', '--template ds0 1000000:value')
-        >>> my_rrd.values = []
-
-        >>> my_rrd.bufferValue('1000000', 'value1', 'value2')
-        >>> my_rrd.bufferValue('1000001', 'value3', 'value4')
-        >>> my_rrd.update(debug=True, dryRun=True)
-        ('somefile', ' 1000000:value1:value2 1000001:value3:value4')
-        >>> my_rrd.update(template='ds1:ds0', debug=True, dryRun=True)
-        ('somefile', '--template ds1:ds0 1000000:value1:value2 1000001:value3:value4')
-        >>> my_rrd.values = []
-
-        >>> my_rrd.bufferValue('1000000:value')
-        >>> my_rrd.bufferValue('1000001:anothervalue')
-        >>> my_rrd.update(debug=True, dryRun=True)
-        ('somefile', ' 1000000:value 1000001:anothervalue')
-        >>> my_rrd.update(template='ds0', debug=True, dryRun=True)
-        ('somefile', '--template ds0 1000000:value 1000001:anothervalue')
-        >>> my_rrd.values = []
-        """
-        values = ':'.join([ str(x) for x in values ])
-        self.values.append((timeOrData, values))
-        self.lastupdate = int(str(timeOrData).split(":")[0])
-
-    # for backwards compatibility
-    bufferValues = bufferValue
-
     def update(self, debug=False, template=None, dryRun=False):
         """
         """
@@ -184,15 +193,15 @@ class RRD(mapper.RRDMapper):
         # sources and values
         self.template = template
         if self.values:
-            data = rrdbackend.prepareObject('update', self)
+            data = self.backend.prepareObject('update', self)
             if debug:
                 print data
             if not dryRun:
-                rrdbackend.update(debug=debug, *data)
+                self.backend.update(debug=debug, *data)
                 self.values = []
 
     def fetch(self, cf="AVERAGE", resolution=None, start=None, end=None,
-              returnStyle="ds"):
+              returnStyle="ds", useBindings=False):
         """
         By default, fetch returns a dict of data source names whose associated
         values are lists. The list for each DS contains (time, data) tuples.
@@ -210,30 +219,40 @@ class RRD(mapper.RRDMapper):
         attributes.resolution = resolution
         attributes.start = start
         attributes.end = end
-        data = rrdbackend.prepareObject('fetch', attributes)
-        return rrdbackend.fetch(*data)[returnStyle]
+        data = self.backend.prepareObject('fetch', attributes)
+        if useBindings:
+            kwds = {"useBindings": useBindings}
+            return self.backend.fetch(*data, **kwds)
+        return self.backend.fetch(*data)[returnStyle]
 
-    def info(self):
+    def info(self, useBindings=False, rawData=False, stream=None):
         """
+        For this method, the info is rendered to stdout, unless rawData is set
+        to True.
         """
-        self.printInfo()
-        for ds in self.ds:
-            ds.printInfo()
-        for index, rra in enumerate(self.rra):
-            rra.printInfo(index)
+        data = self.backend.prepareObject('info', self)
+        kwds = {
+            "useBindings": useBindings, "rawData": rawData, "stream": stream}
+        result = self.backend.info(*data, **kwds)
+        if rawData:
+            return result
 
-    def load(self, filename=None, include_data=False):
+    def load(self, filename=None, includeData=False):
         """
         # Create an empty file:
+        >>> import os, tempfile
+        >>>
         >>> dss = []
         >>> rras = []
-        >>> filename = '/tmp/test.rrd'
+        >>> rrdfile = tempfile.NamedTemporaryFile()
         >>> dss.append(DataSource(dsName='speed', dsType='COUNTER',
         ...   heartbeat=600))
         >>> rras.append(RRA(cf='AVERAGE', xff=0.5, steps=1, rows=24))
         >>> rras.append(RRA(cf='AVERAGE', xff=0.5, steps=6, rows=10))
-        >>> rrd = RRD(filename, ds=dss, rra=rras, start=920804400)
+        >>> rrd = RRD(rrdfile.name, ds=dss, rra=rras, start=920804400)
         >>> rrd.create()
+        >>> os.path.exists(rrdfile.name)
+        True
 
         # Add some values:
         >>> rrd.bufferValue('920805600', '12363')
@@ -246,14 +265,14 @@ class RRD(mapper.RRDMapper):
         # that by passing the "read" mode, were letting the RRD class know that
         # it should call load() immediately, thus giving us read-access to the
         # file's data.
-        >>> rrd2 = RRD(filename, mode="r")
+        >>> rrd2 = RRD(rrdfile.name, mode="r")
 
         # Now let's load the data from self.filename:
         >>> top_level_attrs = rrd2.getData()
         >>> top_level_attrs["lastupdate"]
         920806500
-        >>> top_level_attrs["filename"]
-        '/tmp/test.rrd'
+        >>> top_level_attrs["filename"] == rrdfile.name
+        True
         >>> top_level_attrs["step"]
         300
         >>> len(rrd2.ds)
@@ -273,9 +292,6 @@ class RRD(mapper.RRDMapper):
         >>> rrd.step == rrd2.step
         True
 
-        # Cleanup:
-        >>> import os
-        >>> os.unlink(filename)
         """
         # XXX this should only be enabled once we have the data from the loaded
         # RRD file updating the RRD object
@@ -290,7 +306,7 @@ class RRD(mapper.RRDMapper):
         # is implemented, we will also need to come up with the best way to
         # write this data back to disk (write the individual rows of data that
         # get read in, that is)
-        if include_data:
+        if includeData:
             pass
 
 
@@ -322,7 +338,8 @@ class DataSource(mapper.DSMapper):
     DS:speed:COUNTER:600:U:U
     """
     def __init__(self, dsName=None, dsType=None, heartbeat=None, minval='U',
-        maxval='U', rpn=None):
+                 maxval='U', rpn=None):
+        super(DataSource, self).__init__()
         if dsName == None:
             raise ValueError, "You must provide a name for the data source."
         if dsType == None:
@@ -393,9 +410,9 @@ class RRA(mapper.RRAMapper):
     RRA:AVERAGE:0.5:6:10
     """
     def __init__(self, cf=None, xff=None, steps=None, rows=None, alpha=None,
-        beta=None, seasonal_period=None, rra_num=None, gamma=None,
-        threshold=None, window_length=None, cdpPrepObject=None,
-        databaseObject=None):
+                 beta=None, seasonal_period=None, rra_num=None, gamma=None,
+                 threshold=None, window_length=None, cdpPrepObject=None,
+                 databaseObject=None):
         super(RRA, self).__init__()
         if cf == None:
             msg = "You must provide a value for the consolidation function."
@@ -428,8 +445,10 @@ class RRA(mapper.RRAMapper):
         if self.cf in ['AVERAGE', 'MIN', 'MAX', 'LAST']:
             tail += ':%s:%s:%s' % (self.xff, self.steps, self.rows)
         elif self.cf == 'HWPREDICT':
-            tail += ':%s:%s:%s' % (self.rows, self.alpha, self.beta)
-            tail += ':%s:%s' % (self.seasonal_period, self.rra_num)
+            tail += ':%s:%s:%s:%s' % (
+                self.rows, self.alpha, self.beta, self.seasonal_period)
+            if self.rra_num != None:
+                tail += ':%s' % (self.rra_num)
         elif self.cf == 'SEASONAL':
             tail += ':%s:%s:%s' % (
                 self.seasonal_period, self.gamma, self.rra_num)
@@ -439,8 +458,8 @@ class RRA(mapper.RRAMapper):
         elif self.cf == 'DEVPREDICT':
             tail += ':%s:%s' % (self.rows, self.rra_num)
         elif self.cf == 'FAILURES':
-            tail += ':%s:%s' % (self.rows, self.threshold)
-            tail += ':%s:%s' % (self.window_length, self.rra_num)
+            tail += ':%s:%s:%s:%s' % (
+                self.rows, self.threshold, self.window_length, self.rra_num)
         return main+tail
 
 
@@ -448,11 +467,6 @@ class Query(object):
     pass
 
 
-def _test():
+if __name__ == '__main__':
     from doctest import testmod
     testmod()
-
-
-if __name__ == '__main__':
-    _test()
-
